<?php

namespace Drupal\component_builder\Form;

use Drupal\component_builder\ImportConfigComponent;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ExtensionPathResolver;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Config component item settings.
 */
class ComponentItemSettingsForm extends ConfigFormBase {

  const SETTING = 'component_item.component_item_setting';

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The extension path resolver.
   *
   * @var \Drupal\Core\Extension\ExtensionPathResolver
   */
  protected $extensionPathResolver;

  /**
   * The Import config component services.
   *
   * @var \Drupal\component_builder\ImportConfigComponent
   */
  protected $importConfigComponent;

  /**
   * The field plugin manager service.
   *
   * @var \Drupal\Core\Extension\ModuleHandler
   */
  protected $moduleHandler;

  /**
   * Constructs a new ComponentItemSettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system object.
   * @param \Drupal\Core\Extension\ExtensionPathResolver $extension_path_resolver
   *   The extension path resolver.
   * @param \Drupal\component_builder\ImportConfigComponent $import_config_component
   *   The extension path resolver.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler.
   */
  public function __construct(ConfigFactoryInterface $config_factory, FileSystemInterface $file_system, ExtensionPathResolver $extension_path_resolver, ImportConfigComponent $import_config_component, ModuleHandlerInterface $moduleHandler) {
    parent::__construct($config_factory);
    $this->fileSystem = $file_system;
    $this->extensionPathResolver = $extension_path_resolver;
    $this->importConfigComponent = $import_config_component;
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('file_system'),
      $container->get('extension.path.resolver'),
      $container->get('component_builder.import_config_component'),
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'component_item_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $components = $values['components'];
    $definitions = $form_state->get('definitions');
    $installed = 0;
    foreach ($components as $groupId => $group) {
      foreach ($group as $componentId => $value) {
        $configComponent = $definitions[$componentId];
        if (isset($configComponent['dependencies']) && $value === 1) {
          $dependencies = $configComponent['dependencies'];
          foreach ($dependencies as $module_id => $dependency) {
            if (!$this->moduleHandler->moduleExists($module_id)) {
              $dependencies_message = array_column($dependencies, 'label');
              $dependencies_message =
                implode(',', array_values($dependencies_message));
              $this->messenger()
                ->addError(t('@component_name component requires the @module module. Please install the <a href="@link">@module</a> module before activating this component.',
                  [
                    '@component_name' => $configComponent['label'] ?? '',
                    '@module' => $dependencies_message,
                    '@link' => $dependency['url'],
                  ]));
              unset($components[$groupId][$componentId]);
              break;
            }
          }
        }
      }
    }
    $config = $this->config(self::SETTING);
    foreach ($components as $group) {
      foreach ($group as $componentId => $value) {
        if (!$config->get($componentId) && $value === 1) {
          if (isset($definitions[$componentId])) {
            $id = $definitions[$componentId]['id'];
            $module = $definitions[$componentId]['module'] ?? 'component_builder';
            $label = $definitions[$componentId]['label']->__toString();
            $this->importConfigComponent->createComponentItem($id, $label, $module);
            $installed = 1;
          }
        }
        $config->set($componentId, $value);
      }
    }
    $config->save();
    user_role_grant_permissions(AccountInterface::ANONYMOUS_ROLE, [
      'view component_wrapper',
      'view component_item',
    ]);
    user_role_grant_permissions(AccountInterface::AUTHENTICATED_ROLE, [
      'view component_wrapper',
      'view component_item',
    ]);
    \Drupal::service('cache.render')->invalidateAll();
    if ($installed) {
      $this->messenger()->addMessage('Installed components.');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $plugin_manager = \Drupal::service('plugin.manager.component_builder');
    $definitions = $plugin_manager->getDefinitions();
    $groups = [];
    foreach ($definitions as $component_id => $config) {
      $groups[$config['group']->__toString()][] = $config;
      usort($groups[$config['group']->__toString()], function ($a, $b) {
        return strcmp($a['label']->__toString(), $b['label']->__toString());
      });
    }
    ksort($groups);
    $form_state->set('definitions', $definitions);
    $form['components'] = [
      '#type' => 'details',
      '#title' => t('Components'),
      '#open' => TRUE,
      '#tree' => TRUE,
    ];

    foreach ($groups as $key => $plugins) {
      $group_key = strtolower(str_replace(' ', '-', $key));
      $form['components'][$group_key] = [
        '#type' => 'fieldset',
        '#title' => $key,
      ];
      foreach ($plugins as $key_plugin => $plugin) {
        $dependencies_message = '';
        if (isset($plugin['dependencies'])) {
          $dependencies = $plugin['dependencies'];
          $dependencies_message = array_column($dependencies, 'label');
          $dependencies_message = '<div class="button--danger">(Requires: ' . implode(',', array_values($dependencies_message)) . ') </div>';
        }
        $name = $plugin['label'];
        $id = $plugin['id'];
        $form['components'][$group_key]['wrapper-' . $id] = [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['wrapper-checkbox'],
          ],
        ];
        $form['components'][$group_key]['wrapper-' . $id][$id] = [
          '#type' => 'checkbox',
          '#title' => $dependencies_message ? $name . ' ' . $dependencies_message : $name,
          '#default_value' => $this->config(self::SETTING)->get($id),
          '#attributes' => [
            'class' => ['checkbox-component'],
          ],
          '#parents' => [
            'components',
            $group_key,
            $id,
          ],
        ];
        if ($plugin = $plugin_manager->getInstanceByTemplateName($plugin['id'])) {
          if ($icon = $plugin->getIcon()) {
            $form['components'][$group_key]['wrapper-' . $id]['image'] = [
              '#theme' => 'image',
              '#uri' => $icon,
              '#alt' => $dependencies_message ? $name . ' ' . $dependencies_message : $name,
              '#attributes' => [
                'class' => ['collapse-image', 'visually-hidden'],
              ],
            ];
          }
        }
      }
    }
    $form['#attached']['library'][] = 'component_builder/component_settings';
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTING,
    ];
  }

}
