<?php

namespace Drupal\component_builder\Form;

use Drupal\component_builder\Entity\ComponentWrapperInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ExtensionPathResolver;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Serialization\Yaml;
use Drupal\inline_entity_form\Form\EntityInlineForm;
use Drupal\node\NodeForm;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the inline form for component wrapper.
 */
class ComponentWrapperInlineForm extends EntityInlineForm {

  /**
   * The extension path resolver.
   *
   * @var \Drupal\Core\Extension\ExtensionPathResolver
   */
  protected $extensionPathResolver;

  /**
   * Constructs the inline entity form controller.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity types.
   * @param \Drupal\Core\Extension\ExtensionPathResolver $extension_path_resolver
   *   The extension path resolver.
   */
  public function __construct(EntityFieldManagerInterface $entity_field_manager, EntityTypeManagerInterface $entity_type_manager, ModuleHandlerInterface $module_handler, EntityTypeInterface $entity_type, ExtensionPathResolver $extension_path_resolver) {
    parent::__construct($entity_field_manager, $entity_type_manager, $module_handler, $entity_type);
    $this->extensionPathResolver = $extension_path_resolver;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $container->get('entity_field.manager'),
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $entity_type,
      $container->get('extension.path.resolver'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function entityForm(array $entity_form, FormStateInterface $form_state) {
    $entity_form = parent::entityForm($entity_form, $form_state);

    $header_parents = $entity_form['display_header']['widget']['#parents'];
    $selector_header = $root_header = array_shift($header_parents);
    if ($selector_header) {
      $selector_header = $root_header . '[' . implode('][', $header_parents) . '][value]';
      $entity_form['field_header']['widget'][0]['#states'] = [
        'visible' => [
          ":input[name='$selector_header']" => ['checked' => TRUE],
        ],
      ];
    }

    $footer_parents = $entity_form['display_footer']['widget']['#parents'];
    $selector_footer = $root_footer = array_shift($footer_parents);
    if ($selector_footer) {
      $selector_footer = $root_footer . '[' . implode('][', $footer_parents) . '][value]';
      $entity_form['field_footer']['widget'][0]['#states'] = [
        'visible' => [
          ":input[name='$selector_footer']" => ['checked' => TRUE],
        ],
      ];
    }

    $region_parents = $entity_form['display_in_region']['widget']['#parents'];
    $selector_region = $root_region = array_shift($region_parents);
    if ($selector_region) {
      $selector_region = $root_region . '[' . implode('][', $region_parents) . '][value]';
      $entity_form['field_region']['widget'][0]['#states'] = [
        'visible' => [
          ":input[name='$selector_region']" => ['checked' => TRUE],
        ],
      ];
      $entity_form['field_weight_in_region']['widget'][0]['#states'] = [
        'visible' => [
          ":input[name='$selector_region']" => ['checked' => TRUE],
        ],
      ];

      $region_input = &$entity_form['field_region']['widget'][0]['value'];
      $region_input['#type'] = 'select';
      $theme = \Drupal::config('system.theme')->get('default');
      $region_options = system_region_list($theme);
      $region_input['#options'] = $region_options;
      $region_input['#multiple'] = FALSE;
      unset($region_input['#size']);

      $entity_form['display_in_region_wrapper'] = [
        '#type' => 'details',
        '#title' => $this->t('Region configuration'),
        '#attributes' => [
          'class' => [
            'display-in-region--wrapper',
          ],
        ],
        '#states' => [
          'visible' => [
            ":input[name='$selector_region']" => ['checked' => TRUE],
          ],
        ],
        '#weight' => 1,
        'field_region' => $entity_form['field_region'],
        'field_weight_in_region' => $entity_form['field_weight_in_region'],
      ];
      unset($entity_form['field_region'], $entity_form['field_weight_in_region']);
    }

    $entity_form['wrapper_display'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => [
          'wrapper-display--custom',
        ],
      ],
      'display_title' => $entity_form['display_title'],
      'display_header' => $entity_form['display_header'],
      'display_footer' => $entity_form['display_footer'],
      'display_in_region' => $entity_form['display_in_region'],
    ];
    unset($entity_form['display_header'], $entity_form['display_footer'], $entity_form['display_title'], $entity_form['display_in_region']);
    $form_object = $form_state->getFormObject();
    if ($form_object instanceof NodeForm) {
      $bundle = $form_object->getEntity()->bundle();
      $detect_field = $this->detectEntity();
      if (isset($detect_field[$bundle])) {
        $form_state->set('field_name_alter', $detect_field[$bundle]);
        $form_state->set('template_options', $entity_form['component_type']['widget']['#options']);
        $entity_form['component_attributes'] = [
          '#type' => 'container',
          '#attributes' => ['id' => 'component-attributes-wrapper'],
          '#weight' => $entity_form['component_type']['#weight'],
        ];
        $entity_form['config'] = [
          '#type' => 'details',
          '#title' => t('Config'),
          '#weight' => $entity_form['component_type']['#weight'],
          '#access' => FALSE,
        ];
        $this->loadYmlComponents($entity_form, $form_state);
        $this->renderAttributeMarkup($entity_form, $form_state);
        $entity_form['#attached']['library'][] = 'component_builder/component_wrapper';
      }
    }

    return $entity_form;
  }

  /**
   * {@inheritdoc}
   */
  public function entityFormSubmit(array &$entity_form, FormStateInterface $form_state) {
    $values = $form_state->getValue($entity_form['#parents']);
    $component_id = $values['component_type'][0]['target_id'];
    $component_wrapper_templates = $form_state->get('component_wrapper_templates');
    if ($component_id) {
      $template_machine_name = $component_wrapper_templates[$component_id];
      $display_mode = '';
      $properties = '';
      if ($values['component_attributes'][$template_machine_name]) {
        $properties = json_encode($values['component_attributes'][$template_machine_name]);
      }
      if (isset($values['component_attributes'][$template_machine_name]['properties']['options']['display_modes'])) {
        $display_mode = $values['component_attributes'][$template_machine_name]['properties']['options']['display_modes'];
      }
      $form_state->setValue(array_merge($entity_form['#parents'], ['field_properties']), [0 => ['value' => $properties]]);
      $form_state->setValue(array_merge($entity_form['#parents'], ['field_display_mode']), [0 => ['value' => $display_mode]]);
    }
    parent::entityFormSubmit($entity_form, $form_state);
  }

  /**
   * Set value for attribute of component.
   */
  public function renderAttributeMarkup(array &$entity_form, FormStateInterface $form_state) {
    $trigger = $form_state->getTriggeringElement();
    if (isset($trigger['#ief_row_delta'])) {
      $delta = $trigger['#ief_row_delta'];
      $component_wrapper = $form_state->get(
        [
          'inline_entity_form',
          $entity_form['#ief_id'],
          'entities',
          $delta,
          'entity',
        ]
      );
      if (isset($entity_form['component_attributes'])) {
        if ($component_wrapper instanceof ComponentWrapperInterface) {
          $component_id = $component_wrapper->get('component_type')->getValue()[0]['target_id'];
          $component_wrapper_templates = $form_state->get('component_wrapper_templates');
          $template_machine_name = $component_wrapper_templates[$component_id];

          $properties = $component_wrapper->get('field_properties')->value;
          if ($properties) {
            $properties = json_decode($properties, TRUE);
            if (is_array($properties)) {
              foreach ($properties as $key => $property_types) {
                foreach ($property_types as $parent_type_key => $types) {
                  foreach ($types as $type_key => $value) {
                    $entity_form['component_attributes'][$template_machine_name][$key][$parent_type_key][$type_key]['#default_value'] = $value;
                  }
                }
              }
            }
          }
          $display = $component_wrapper->get('field_display_mode')->value;
          if ($display) {
            $entity_form['component_attributes'][$template_machine_name]['display_mode']['#default_value'] = json_decode($display, TRUE);
          }
        }
      }
    }
  }

  /**
   * Render attribute as markup.
   */
  public function loadYmlComponents(array &$entity_form, FormStateInterface $form_state) {
    $field_name_alter = $form_state->get('field_name_alter');
    if (is_array($field_name_alter)) {
      $templates = [];
      foreach ($field_name_alter as $field_name) {
        if (in_array($field_name, $entity_form['#parents'])) {
          $options = $form_state->get('template_options');
          unset($options['_none']);
          foreach ($options as $template_id => $component_type) {
            $component_type_name = strtolower($component_type);
            $component_type_machine = preg_replace('@[^a-z0-9-]+@', '_', $component_type_name);
            $templates[$template_id] = $component_type_machine;
            $plugin_manager = \Drupal::service('plugin.manager.component_builder');
            if ($plugin = $plugin_manager->getInstanceByTemplateName($component_type_machine)) {
              $yml_path = $plugin->getDefinePath();
              if ($yml_path && file_exists($yml_path)) {
                $yml_content = file_get_contents($yml_path);
                $yml_content = Yaml::decode($yml_content);
                if (isset($yml_content['properties'])) {
                  $properties_component = $yml_content['properties'];
                  if (is_array($properties_component)) {
                    $entity_form['component_attributes'][$component_type_machine] = [
                      '#type' => 'container',
                      '#attributes' => [
                        'id' => 'component-attributes-wrapper--' . $component_type_machine,
                        'class' => [
                          'component-attributes-wrapper--' . $component_type_machine,
                          'component-attributes-wrapper--item',
                        ],
                      ],
                    ];
                  }
                  foreach ($properties_component as $key_properties_component => $properties) {
                    $title_properties = $key_properties_component;
                    if ($properties['label']) {
                      $title_properties = $properties['label'];
                      unset($properties['label']);
                    }
                    $entity_form['component_attributes'][$component_type_machine][$key_properties_component] = [
                      '#type' => 'details',
                      '#attributes' => [
                        'id' => $component_type_machine . '-' . $key_properties_component,
                        'class' => [
                          $component_type_machine . '-' . $key_properties_component . '--item',
                          $component_type_machine . '-' . $key_properties_component,
                        ],
                      ],
                      '#title' => $title_properties,
                    ];
                    foreach ($properties as $key => $values) {
                      if (is_array(reset($values))) {
                        $entity_form['component_attributes'][$component_type_machine][$key_properties_component][$key] = [
                          '#type' => 'fieldset',
                          '#attributes' => [
                            'id' => $component_type_machine . '--' . $key_properties_component . '-' . $key,
                            'class' => ['component-attributes-wrapper--' . $key_properties_component . '-' . $key],
                          ],
                          '#title' => $key,
                        ];
                        foreach ($values as $option_key => $option_value) {
                          $options = ['_none' => '-- None --'];
                          $options += $option_value;
                          $entity_form['component_attributes'][$component_type_machine][$key_properties_component][$key][$option_key] = [
                            '#type' => 'select',
                            '#title' => 'Select ' . $option_key,
                            '#options' => $options,
                          ];
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
      $form_state->set('component_wrapper_templates', $templates);
    }
  }

  /**
   * Detect entity has field references to component wrapper.
   */
  public function detectEntity() {
    static $drupal_static_fast_detect_entity_id;
    if (!isset($drupal_static_fast_detect_entity_id)) {
      $storage = $this->entityTypeManager->getStorage('field_storage_config');
      $entity_ids = $storage->getQuery()
        ->accessCheck(TRUE)
        ->execute();
      $field_storages = $storage->loadMultipleOverrideFree($entity_ids);
      $drupal_static_fast_detect_entity_id = [];
      foreach ($field_storages as $field_storage) {
        $field_storage_name = $field_storage->getName();
        $filed_storage_type = $field_storage->getType();
        $allowed_type = [
          'entity_reference',
        ];
        if (in_array($filed_storage_type, $allowed_type, TRUE)) {
          $filed_storage_target_type = $field_storage->getSettings()['target_type'];
          if ($filed_storage_target_type === 'component_wrapper') {
            foreach ($field_storage->getBundles() as $bundle) {
              $drupal_static_fast_detect_entity_id[$bundle][] = $field_storage_name;
            }
          }
        }
      }
    }
    return $drupal_static_fast_detect_entity_id;
  }

}
